function shPlotFunction( band_max, sh_coeffs, level_vis, new_figure) 
%
% shPlotFunction( band_max, sh_coeffs,  level_vis, new_figure) 
%
% Visualize signal represented in Spherical Harmonics.
%
% @param  band_max   maximum band of the Spherical Harmonics signal
% @param  sh_coeffs  Spherical Harmonics basis function 
% @param  level_vis  parameter to control the resolution of the spherical
%                    subdivision
% @param  new_figure create a new figure for the figure (0 or 1)

  % default parameters
  defval( 'new_figure', 1);
  defval( 'level_vis', 4);
  
  % visulization based on subdivided platonic solid
  % try to load precomputed forest, compute if it fails
  fname_forest= ['visualization/forest_octahedron_l' num2str(level_vis) '.mat'];    
  try 
    load( fname_forest);
  catch
    forest = getForestPlatonicSolid( 'octahedron', level_vis, 0);
    save( fname_forest, 'forest');
  end
  
  % collect all vertices
  verts = [];
  colors  = [];
  T = [];
  fname_base = ['visualization/vis_data_octahedron_', ...
                'L' num2str(band_max), '_l' num2str(level_vis)]; 
    
  for( i = 1 : numel( forest))
   
    % basic visualization data for current octand, try to load and compute
    % if loading fails
    fname = [fname_base '_' num2str(i) '.mat'];
    try 
      load( fname);
    catch 
            
      % verbose
      if( 1 == i)
        disp( sprintf( ['Computing visualization data. ', ...
                        'This may take a few minutes.']));
      end
                    
      % get the basic structure out of the platonic solid
      v = []; c = [];  connect = [];
      [v , connect, centroids] = collectVertsColors( forest(i), level_vis, ...
                                                     v, c, connect);
      y_lm = shFunctionVals( band_max, centroids);
      save( fname, 'v', 'connect', 'centroids', 'y_lm');
    end
    
    % reconstruct function at the centroids and replicate the color for all
    % three vertices
    f_vals = abs( transpose( y_lm * sh_coeffs')) ./ 255;
    f_vals = repmat( f_vals, 3, 1);
    f_vals = f_vals(:);
    f_vals = reshape( f_vals, 3, numel(f_vals)/3);
    
    % store data for current quadrant
    T = [T ; connect + size(verts,2)];
    verts = [verts v];
    colors = [colors f_vals];
    
  end
  
  % setup plotting
  if( 1 == new_figure)
    figure;
  end
  axis equal;  
  axis off;
  hold on;   
  
  % plot data
  trimesh( T, verts(1,:), verts(2,:), verts(3,:), ...
           'FaceVertexCData', colors', 'FaceColor', 'interp', ...
           'EdgeColor', 'none');
         
  view(-75,5);  
           
end
         
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [verts , connect, centroids] = collectVertsColors( tri, level, ...
                                                    verts, connect, centroids)

  if( getLevel(tri) == level)
      
    cent = r3_to_s2( getCentroidSTri( tri));
    centroids( getIndexLevel(tri)+1,:) = cent';
    
    index = (3*getIndexLevel(tri)) + 1;
    
    % store connectivity
    connect(getIndexLevel(tri)+1, :) = [index + 2 , index + 1 , index];
    
    % store vertices
    verts( : , index : index + 2) = getVertsEC( tri);
  
  else

    % recursively traverse the tree
    childs = getChilds( tri);
    
    for( i = 1 : numel( childs))
      [verts , connect, centroids] = collectVertsColors(childs(i), level, ...
                                                     verts, connect, centroids);
    end

  end

end
