function rcoeffs = shRotateBlanco( band_max, coeffs, R)
% 
% r_coeffs = shRotateBlanco( band_max, coeffs, R);
%
% Rotate spherical harmonics coefficients using recurrance scheme
% for efficient calculation of rotation matrix, based on method
% of Blanco et al. [1997].
%
% @return rcoeffs     the rotated SH coeffs
% @param  band_max     maximum SH band
% @param  R            rotation (as matrix)
 
    tic

    angles=[0 0 0];
    [angles(1), angles(2), angles(3)]=getRotationEulerZYZMatrix(R);
    angles=deg_to_rad(angles);
    
    alpha = -angles(1);
    beta = -angles(2);
    gamma = -angles(3);
    
    rcoeffs=coeffs;
    
    % Cell array 'd' will store wigner d matrices
    d={[1]};
    
    % sin/cos lookup tables, to be calculated
    % from recurrence formulas as we iterate
    sin_alpha=zeros(1,band_max);
    cos_alpha=ones(1,band_max);
    sin_gamma=zeros(1,band_max);
    cos_gamma=ones(1,band_max);


    % Band zero invariant under rotation
    rcoeffs(1) = coeffs(1);

    % precalculate some constants and initial seed values
    root2=sqrt(2);    
    sin_alpha1 = sin(alpha);
    cos_alpha1 = cos(alpha);
    sin_gamma1 = sin(gamma);
    cos_gamma1 = cos(gamma);
    
    for L=1:band_max
        % calculate sin/cos lookup table entries for this band
        sin_alpha(L+1) = sin_alpha1 * cos_alpha(L) + cos_alpha1 * sin_alpha(L);
        cos_alpha(L+1) = cos_alpha1 * cos_alpha(L) - sin_alpha1 * sin_alpha(L);

        sin_gamma(L+1) = sin_gamma1 * cos_gamma(L) + cos_gamma1 * sin_gamma(L);
        cos_gamma(L+1) = cos_gamma1 * cos_gamma(L) - sin_gamma1 * sin_gamma(L);               
        
        % calculate wigner d matrix for band L, d{L+1} using recurrance on d{L} and d{L-1}
        if (L==1)
            d{L+1} = wigner(L,-angles);
        else        
            d{L+1}=wigner(L,-angles,d{L},d{L-1});
            % free memory for wigner matrix that's no longer needed
            d{L-1} = [];
        end
        
        % Calculate rotation matrix for band L
        R=zeros(2*L+1,2*L+1);
        for m=-L:L
            for mp=-L:L
                s1 = sign(mp);
                if s1==0 s1=1; end
                s2 = sign(m);
                if s2==0 s2=1; end

                absm=abs(m);
                absmp=abs(mp);
                
                d1=d{L+1}(absmp+1+L, absm+1+L);
                d2=(-1)^m * d{L+1}(absm+1+L, -absmp+1+L);
                
                if (m < 0)
                    omeg_m_alpha1 = root2 * sin_alpha(absm+1);
                    omeg_m_alpha2 = root2 * cos_alpha(absm+1);
                elseif (m==0)
                    omeg_m_alpha1 = 1;
                    omeg_m_alpha2 = 1;
                elseif (m > 0)
                    omeg_m_alpha1 = root2 * cos_alpha(absm+1);
                    omeg_m_alpha2 = root2 * sin_alpha(absm+1);                    
                end

                if (mp < 0)
                    omeg_mp_gamma1 = root2 * sin_gamma(absmp+1);
                    omeg_mp_gamma2 = root2 * cos_gamma(absmp+1);                    
                elseif (mp==0)
                    omeg_mp_gamma1 = 1;
                    omeg_mp_gamma2 = 1;
                elseif (mp > 0)
                    omeg_mp_gamma1 = root2 * cos_gamma(absmp+1);
                    omeg_mp_gamma2 = root2 * sin_gamma(absmp+1);
                end
                
                R(mp+L+1,m+L+1) = s1 * omeg_m_alpha1 * omeg_mp_gamma1 * (d1+d2)/ 2.0 ...
                     - s2 * omeg_m_alpha2 * omeg_mp_gamma2 * (d1-d2)/2.0;
            end
        end
        
        % Multiply coefficients in band L by the rotation matrix
        % Need to calculate index into linear vectors 'rcoeffs' and 'coeffs'
        index = L^2+1  ;
        num_coeffs = 2*L+1;
        rcoeffs(:,index:index+num_coeffs-1) = transpose( R * coeffs(:,index:index+num_coeffs-1)');

    end

    e=toc;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function D = wigner( L, angles, dn1,dn2)
    % Calculate wigner small d matrix through recurrence relations
    % Requires previous two bands L-1 and L-2 

    defval('d',{[]})
    defval('dn1',[])
    defval('dn2',[])

    % Name the Euler angles, to coincide with notation in Blanco's paper
    alpha = angles(1);
    beta = angles(2);
    gamma = angles(3);

    % d{X} contains the wigner-d matrix for SH band X
    % The first two bands can be calculated analytically

    if (L==0)
        D=[1];
        return
    end

    if (L==1)
        D = [ cos(beta/2)^2          sqrt(1/2)*sin(beta)          sin(beta/2)^2
             -1/sqrt(2)*sin(beta)    cos(beta)                  1/sqrt(2)*sin(beta)
              sin(beta/2)^2          -sqrt(1/2)*sin(beta)        cos(beta/2)^2];
        return
    end

    tanhalfbeta = tan(beta/2);
    coshalfbeta = cos(beta/2);
    cosbeta = cos(beta);

    l=L;
    for m=0:l-2
        for mp = -m:m
            % eq 64
            %x = l*(2*l-1) /sqrt((l^2-m^2)*(l^2-mp^2));
            %t1 =(dat(d,1,0,0)- (m*mp)/(l*(l-1))) * dat(d,l-1,m,mp);           
            %t2 = (sqrt(((l-1)^2 - m^2) * ((l-1)^2 - mp^2))/ ((l-1) * (2*l-1))) * dat(d,l-2,m,mp);
            %x= x * (t1-t2);

            x = l*(2*l-1) /sqrt((l^2-m^2)*(l^2-mp^2));
            t1 =(cosbeta - (m*mp)/(l*(l-1))) * dn1(m+l,mp+l);
            t2 = (sqrt(((l-1)^2 - m^2) * ((l-1)^2 - mp^2))/ ((l-1) * (2*l-1))) * dn2(m+l-1,mp+l-1);
            x= x * (t1-t2);            

            D(m+l+1,mp+l+1) = x;
        end
    end

    % eq 65
    D(l+l+1,l+l+1) = coshalfbeta^(2*l);
    % eq 66
    D(l-1+l+1,l-1+l+1) = (l * cosbeta - l +1) * coshalfbeta^(2*(l-1));

    % eq 67
    for mp=l-1:-1:-l
        %d{l+1}(l + l +1, mp + l + 1) = - sqrt((l+(mp+1))/(l-(mp+1)+1)) *
        %tanhalfbeta * dat(d,l,l,mp+1);       
        D(l + l +1, mp + l + 1) = - sqrt((l+(mp+1))/(l-(mp+1)+1)) * tanhalfbeta * D(l+l+1,l+mp+2);
    end

    % eq 68
    for mp = l-2:-1:1-l
    %        d{l+1}(l-1 + l +1, mp + l +1) = -(l*cosbeta-(mp+1)+1)/(l*cosbeta-(mp+1)) * ...
    %            sqrt((l+(mp+1))/(l-(mp+1)+1)) * tanhalfbeta *
    %            dat(d,l,l-1,mp+1);

        D(l-1 + l +1, mp + l +1) = -(l*cosbeta-(mp+1)+1)/(l*cosbeta-(mp+1)) * ...
            sqrt((l+(mp+1))/(l-(mp+1)+1)) * tanhalfbeta * ...
            D(l+l,l+mp+2);

    end

    % identities for filling in remaining required entries
    for m=0:L
        for mp=0:m-1
            D(mp+L+1,m+L+1)=(-1)^(mp+m)*D(m+L+1,mp+L+1);
            D(mp+L+1,-m+L+1)= D(m+L+1,-mp+L+1);        
        end 
    end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function x=dat(d,l,m,mp)
    % indexes wigner d matrix for band l similar to notation
    % used in Blanco's paper
    % 
    % this was used for clarity during debugging
    % no longer used for efficienty reasons
    x = d{l+1}(m+l+1,mp+l+1);
end
