function lambda = getPointsS2( total_samples, method, theta )
%
% lambda = getPointsS2( total_samples, method, theta)
%
% Create set of points on the sphere.
% 
% @return  lambda       total_samples points on the sphere in standard 
%                       spherical coordinates (theta,phi)
% @param   num_samples  Number of random points to generate
% @param   method       Method for generating points on the sphere ('halton', 
%                       'spiral', 'random', 'equator', 'semi-equator'; 
%                       default is 'halton')
% @param theta         'equi-latitude': latitude for generating equi-latitude 

  defval( 'method', 'halton');
  defval( 'theta', pi/3);

  switch method
    case 'halton'
      lambda=halton(total_samples,2);
      lambda(:,1) = pi/2 - asin(lambda(:,1));
      lambda(:,2) = 2 * pi * lambda(:,2);
    case 'spiral'
      lambda = spiralPoints( total_samples+2);
      % skip the poles which can lead to points where Spherical Harmonics
      % cannot be evaluated accurately when used with optimization
      lambda = lambda( 2:end-1, :);
    case 'spiral-with-pole'
      lambda = spiralPoints( total_samples+1);
      % skip only the last point which is always the north pole and
      % combined with the first point of the sequence which is the south
      % pole would lead to two identical kernels
      lambda = lambda( 1:end-1, :);
    case 'equator'
      % choose points on the equator
      delta = 2*pi / total_samples;
      lambda = zeros( total_samples, 2);
      lambda(:,1) = pi/2;
      epsilon = 10^-6;
      lambda(:,2) = [0 : delta : (2*pi-epsilon)];
    case 'semi-equator'
      defval( 'param', pi/3);
      % choose points on the equator
      delta = 2*pi / total_samples;
      lambda = ones( total_samples, 2);
      lambda(:,1) = theta;
      epsilon = 10^-6;
      lambda(:,2) = [0 : delta : (2*pi-epsilon)];      
    case 'random'
      % 5489 sets random number generator to default value
      rand('twister', 5489);
      for i=1:total_samples
          lambda(i,:) = rand(1,2);
      end
      lambda(:,1) = lambda(:,1) * pi;
      lambda(:,2) = lambda(:,2) * 2 * pi;
    case 'cluster1'
      % points in subset of top hemisphere
      lambda=halton(total_samples,2);      
      lambda(:,1) = (8/10) * (pi/2 - asin(lambda(:,1)));
      lambda(:,2) = lambda(:,2) * 2 * pi;
    case 'cluster2'
      % points in subset of top hemisphere
      lambda=halton(total_samples,2);      
      lambda(:,1) = (7/10) * (pi/2 - asin(lambda(:,1)));
      lambda(:,2) = lambda(:,2) * 2 * pi;
    case 'cluster3'
      % points in subset of top hemisphere
      lambda=halton(total_samples,2);      
      lambda(:,1) = (2/5) * (pi/2 - asin(lambda(:,1)));
      lambda(:,2) = lambda(:,2) * 2 * pi;
    otherwise
      error('Unknown method "%s".', method);
  end
  
end