function points = halton( k, dim)
%
% points = halton( k, dim)
%
% Create k points of a two-dimensional Halton sequence.
%
% @return points  Halton points in R^2.
% @param  k       Number of points in the sequence.
% @param  dim     dimension of points

  points = zeros( k, dim);

  % find the prime seeds for the requested dimensions
  if( 2 == dim)
    prime_seeds = [2, 3];
  elseif( 3 == dim)
    prime_seeds = [2, 3, 5];    
  else
    % Use the prime number theorem to find dim prime numbers. Given a prime
    % number count n we want to solve for the number which is likely to
    % have at least n primes which are smaller. Solving the "inverse prime
    % number theorem" requires the LambertW function (also known as
    % ProductLog) which is however not available or Matlab (and the available
    % implementations behave differently than the Mathematica one).
    % Try stupid estimate which will work for small dim
    n = 100 * dim;
    prime_seeds = primes( n);
    if( numel( prime_seeds) < dim)
      error( 'Failed to find enough prime numbers.');
    end
    prime_seeds = prime_seeds( 1:dim);
  end
  
  % The second argument, 2 and 3, are the first and second prime (for the
  % first and second dimension). For n dimensions the first n points have
  % to be used.
  % The first point generated by the implementation we use is (0,0) which
  % we discard since it's at the fringe of the domain.
  for( i = 1 : dim) 
    temp = vdcorput( k, prime_seeds(i));
    points(:,i) = temp(2:end)';
  end
  
end